import os
import pytest
import collections
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

import wcs.qommon.ctl
from wcs.ctl.collectstatic import CmdCollectStatic
from wcs.ctl.process_bounce import CmdProcessBounce

from utilities import create_temporary_pub, clean_temporary_pub

@pytest.fixture
def pub():
    return create_temporary_pub()

def teardown_module(module):
    clean_temporary_pub()

def test_loading():
    ctl = wcs.qommon.ctl.Ctl(cmd_prefixes=['wcs.ctl'])
    ctl.load_all_commands(ignore_errors=False)
    assert 'start' in ctl.get_commands().keys()
    # call all __init__() methods
    for cmd in ctl.get_commands().values():
        cmd()

def test_collectstatic(pub):
    CmdCollectStatic.collectstatic(pub)
    assert os.path.exists(os.path.join(pub.app_dir, 'collectstatic', 'css', 'wcs.css'))
    assert os.path.exists(os.path.join(pub.app_dir, 'collectstatic', 'css', 'qommon.css'))
    assert os.path.exists(os.path.join(pub.app_dir, 'collectstatic', 'css', 'gadjo.css'))
    assert os.path.exists(os.path.join(pub.app_dir, 'collectstatic', 'xstatic', 'jquery.js'))
    CmdCollectStatic.collectstatic(pub, clear=True, link=True)
    assert os.path.islink(os.path.join(pub.app_dir, 'collectstatic', 'css', 'wcs.css'))

def test_get_bounce_addrs():
    msg = MIMEText('Hello world')
    assert CmdProcessBounce.get_bounce_addrs(msg) is None

    msg = MIMEMultipart(_subtype='mixed')
    msg.attach(MIMEText('Hello world'))
    msg.attach(MIMEText('<p>Hello world</p>', _subtype='html'))
    assert CmdProcessBounce.get_bounce_addrs(msg) is None

    msg = MIMEText('Hello world')
    msg['x-failed-recipients'] = 'foobar@localhost'
    assert CmdProcessBounce.get_bounce_addrs(msg) == ['foobar@localhost']

    msg = MIMEText('''failed addresses follow:
                      foobar@localhost
                      message text follows:''')
    assert CmdProcessBounce.get_bounce_addrs(msg) == ['foobar@localhost']

def test_wipe_formdata(pub):
    from wcs import formdef, fields
    from wcs.ctl.wipe_data import CmdWipeData

    form_1 = formdef.FormDef()
    form_1.name = 'example'
    form_1.fields = [fields.StringField(id='0', label='Your Name'),
                      fields.EmailField(id='1', label='Email')]
    form_1.store()
    form_1.data_class().wipe()
    formdata_1 = form_1.data_class()()

    formdata_1.data = {'0': 'John Doe', '1': 'john@example.net'}
    formdata_1.store()

    assert form_1.data_class().count() == 1

    form_2 = formdef.FormDef()
    form_2.name = 'example2'
    form_2.fields = [fields.StringField(id='0', label='First Name'),
                    fields.StringField(id='1', label='Last Name')]
    form_2.store()
    form_2.data_class().wipe()
    formdata_2 = form_2.data_class()()
    formdata_2.data = {'0': 'John', '1': 'Doe'}
    formdata_2.store()
    assert form_2.data_class().count() == 1

    wipe_cmd = CmdWipeData()
    sub_options_class = collections.namedtuple('Options', ['all'])
    sub_options = sub_options_class(False)

    # test with no options
    wipe_cmd.wipe(pub, sub_options, [])
    assert form_1.data_class().count() == 1
    assert form_2.data_class().count() == 1

    # wipe one form formdatas
    wipe_cmd.wipe(pub, sub_options, [form_1.url_name])
    assert form_1.data_class().count() == 0
    assert form_2.data_class().count() == 1

    # wipe all formdatas
    sub_options = sub_options_class(True)
    wipe_cmd.wipe(pub, sub_options, [])
    assert form_1.data_class().count() == 0
    assert form_2.data_class().count() == 0
